package cn.itcast.hotel;

import org.apache.http.HttpHost;
import org.elasticsearch.action.search.SearchRequest;
import org.elasticsearch.action.search.SearchResponse;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.SearchHit;
import org.elasticsearch.search.SearchHits;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightBuilder;
import org.elasticsearch.search.fetch.subphase.highlight.HighlightField;
import org.elasticsearch.search.sort.SortOrder;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import java.io.IOException;
import java.util.Map;

public class HotelSearchTest {
    private RestHighLevelClient client;

    @BeforeEach
    void setup() {
        this.client = new RestHighLevelClient(RestClient.builder(
                // HttpHost.create("http://192.168.1.4:9200"))
                HttpHost.create("http://192.168.3.98:9200")
        ));
    }

    @AfterEach
    void teardown() throws Exception {
        this.client.close();
    }

    // 查询文档
    @Test
    void testMatchAll() throws IOException {
        // 1. 准备request
        SearchRequest request = new SearchRequest("hotel");
        // 2. 准备DSL
        request.source().query(QueryBuilders.matchAllQuery());
        // 3.发送请求
        SearchResponse response = client.search(request, RequestOptions.DEFAULT);
        // 解析结果
        SearchHits searchHits = response.getHits();
        long total = searchHits.getTotalHits().value;
        // 总数
        System.out.println("total===>" + total);
        searchHits.forEach(hit -> {
            // 可以将这个转成酒店的对象
            String json = hit.getSourceAsString();
            System.out.println(json);
        });
    }

    // 根据字段查询
    @Test
    void testMatch() throws Exception {
        // 准备Request
        SearchRequest request = new SearchRequest("hotel");
        // 准备DSL
        request.source().query(QueryBuilders.matchQuery("all", "如家"));
        // 发起请求
        SearchResponse response = client.search(request, RequestOptions.DEFAULT);
        // 解析响应
        SearchHits searchHits = response.getHits();
        // 得到总数
        long total = searchHits.getTotalHits().value;
        System.out.println("total===>" + total);
        SearchHit[] hits = searchHits.getHits();
        for (SearchHit hit : hits) {
            System.out.println(hit);
        }
    }

    // 根据范围查询
    @Test
    void testBool() throws Exception {
        // 1. 准备request
        SearchRequest request = new SearchRequest("hotel");
        // 准备DSL
        BoolQueryBuilder boolQuery = QueryBuilders.boolQuery();
        boolQuery.must(QueryBuilders.termQuery("city", "上海"));
        // 添加range
        boolQuery.filter(QueryBuilders.rangeQuery("price").lte(250));
        request.source().query(boolQuery);

        SearchResponse response = client.search(request, RequestOptions.DEFAULT);
        SearchHits searchHits = response.getHits();

        // 得到总数
        long total = searchHits.getTotalHits().value;
        System.out.println("total===>" + total);
        searchHits.forEach(hit -> {
            // 可以将这个转成酒店的对象
            String json = hit.getSourceAsString();
            System.out.println(json);
        });
    }

    // 排序查询
    @Test
    void testPageSort() throws Exception {
        SearchRequest request = new SearchRequest("hotel");
        request.source().query(QueryBuilders.matchAllQuery());
        request.source().sort("price", SortOrder.ASC);
        request.source().from(0).size(5);
        SearchResponse response = client.search(request, RequestOptions.DEFAULT);

        SearchHits searchHits = response.getHits();
        searchHits.forEach(hit -> {
            String json = hit.getSourceAsString();
            System.out.println(json);
        });
    }

    // 高亮显示
    @Test
    void testHighlight() throws Exception {
        // 准备request
        SearchRequest request = new SearchRequest("hotel");
        // 准备DSL
        request.source().query(QueryBuilders.matchQuery("all", "如家"));
        // 高亮
        request.source().highlighter(new HighlightBuilder().field("name").requireFieldMatch(false));
        // 发送请求
        SearchResponse response = client.search(request, RequestOptions.DEFAULT);
        // 解析响应
        SearchHits searchHits = response.getHits();
        searchHits.forEach(hit -> {
            String json = hit.getSourceAsString();
            System.out.println(json);

            // 得到高亮
            Map<String, HighlightField> map = hit.getHighlightFields();
            HighlightField highlightField = map.get("name");
            System.out.println("name===>" + highlightField.getName());
            System.out.println("fragments===>" + highlightField.getFragments()[0]);
        });
    }
}
